/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import edu.mit.spacenet.Preferences;
import edu.mit.spacenet.io.XStreamEngine;

/**
 * A dialog to edit the global SpaceNet preferences.
 * 
 * @author Paul Grogan
 */
public class PreferencesDialog extends JDialog {
	private static final long serialVersionUID = -802497885876127342L;
	
	private SpaceNetFrame spaceNetFrame;
	private Preferences preferences;
	private JCheckBox autoRefreshCheck;
	private JTextField defaultDirectoryText;
	
	/**
	 * The constructor.
	 * 
	 * @param spaceNetFrame the parent SpaceNet frame
	 */
	public PreferencesDialog(SpaceNetFrame spaceNetFrame) {
		super(spaceNetFrame, "SpaceNet Preferences");
		this.spaceNetFrame = spaceNetFrame;
		preferences = spaceNetFrame.getPreferences();
		
		buildDialog();
	}
	private void buildDialog() {
		JPanel contentPanel = new JPanel();
		contentPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.LINE_END;
		contentPanel.add(new JLabel("Default directory: "), c);
		c.gridx++;
		c.weightx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.LINE_START;
		defaultDirectoryText = new JTextField(preferences.getDefaultDirectory());
		defaultDirectoryText.setPreferredSize(new Dimension(300,20));
		contentPanel.add(defaultDirectoryText, c);
		c.gridx++;
		c.weightx = 0;
		JButton browseButton = new JButton("Browse...", new ImageIcon(getClass().getClassLoader().getResource("icons/folder_explore.png")));
		browseButton.setMargin(new Insets(3,3,3,3));
		browseButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fileChooser = new JFileChooser(preferences.getDefaultDirectory());
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				int returnVal = fileChooser.showOpenDialog(getThis());
				if(returnVal == JFileChooser.APPROVE_OPTION) {
					defaultDirectoryText.setText(fileChooser.getSelectedFile().getAbsolutePath());
				}
			}
		});
		contentPanel.add(browseButton, c);
		c.gridy++;
		c.gridx = 0;
		c.gridwidth = 3;
		autoRefreshCheck = new JCheckBox("Auto-refresh charts (lags with large scenarios)");
		autoRefreshCheck.setSelected(preferences.isAutoRefresh());
		contentPanel.add(autoRefreshCheck, c);
		
		c.gridy++;
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 3, 0));
		JButton okButton = new JButton("OK");
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				savePreferences();
				dispose();
			}
		});
		buttonPanel.add(okButton, c);
		getRootPane().setDefaultButton(okButton);
		JButton cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		buttonPanel.add(cancelButton, c);
		contentPanel.add(buttonPanel, c);
		

		setModal(true);
		setContentPane(contentPanel);
	}
	private void savePreferences() {
		preferences.setAutoRefresh(autoRefreshCheck.isSelected());
		preferences.setDefaultDirectory(defaultDirectoryText.getText().equals("")?null:defaultDirectoryText.getText());
		XStreamEngine.writePreferences(preferences);
		spaceNetFrame.updateView();
	}
	private PreferencesDialog getThis() {
		return this;
	}
	
	/**
	 * Creates and displays the preferences dialog.
	 * 
	 * @param spaceNet the parent SpaceNet frame
	 */
	public static void createAndShowGUI(SpaceNetFrame spaceNet) {
		PreferencesDialog dialog = new PreferencesDialog(spaceNet);
		dialog.pack();
		dialog.setLocationRelativeTo(null);
		dialog.setResizable(false);
		dialog.setVisible(true);
	}
}
