/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.command;

import javax.swing.JOptionPane;

import edu.mit.spacenet.gui.data.DataSourceDialog;
import edu.mit.spacenet.scenario.data.DataSourceType;
import edu.mit.spacenet.scenario.data.Spreadsheet_2_5;

/**
 * The command to change the data source of a scenario.
 * 
 * @author Paul Grogan
 */
public class ChangeDataSourceCommand implements I_Command {
	private DataSourceDialog dataSourceDialog;
	private DataSourceType dataSourceType;
	
	/**
	 * The constructor.
	 * 
	 * @param dataSourceDialog the data source dialog component
	 * @param dataSourceType the new data source type
	 */
	public ChangeDataSourceCommand(DataSourceDialog dataSourceDialog, DataSourceType dataSourceType) {
		this.dataSourceDialog = dataSourceDialog;
		this.dataSourceType = dataSourceType;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.command.I_Command#execute()
	 */
	public void execute() {
		int answer = JOptionPane.YES_OPTION;
		if(dataSourceDialog.getScenarioPanel().getScenario().getDataSource()!=null) {
			answer = JOptionPane.showConfirmDialog(dataSourceDialog, 
					"Changing the data source will reset all data. Continue?",
					"Confirm Data Source Change", JOptionPane.YES_NO_OPTION);
		}
		if(answer==JOptionPane.YES_OPTION) {
			dataSourceDialog.dispose();
			
			switch(dataSourceType) {
			case EXCEL_2_5:
				dataSourceDialog.getScenarioPanel().getScenario().setDataSource(new Spreadsheet_2_5(null));
				break;
			case NONE:
				dataSourceDialog.getScenarioPanel().getScenario().setDataSource(null);
				break;
			}
			
			dataSourceDialog.getScenarioPanel().updateView();
			
			EditDataSourceCommand command = new EditDataSourceCommand(dataSourceDialog.getScenarioPanel());
			command.execute();
		} else {
			dataSourceDialog.updateView();
		}
	}
}