/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.command;

import java.awt.Cursor;
import java.util.ArrayList;
import java.util.List;

import org.jdesktop.swingworker.SwingWorker;

import edu.mit.spacenet.domain.network.Location;
import edu.mit.spacenet.domain.network.edge.Edge;
import edu.mit.spacenet.domain.network.node.Node;
import edu.mit.spacenet.gui.ScenarioPanel;
import edu.mit.spacenet.gui.SpaceNetFrame;

/**
 * The command to filter the scenario network based on its scenario type. Any
 * nodes or edges that are in use will not be removed by this operation.
 * 
 * @author Paul Grogan
 */
public class FilterNetworkCommand implements I_Command {
	private ScenarioPanel scenarioPanel;
	
	/**
	 * The constructor.
	 * 
	 * @param scenarioPanel the scenario panel component
	 */
	public FilterNetworkCommand(ScenarioPanel scenarioPanel) {
		this.scenarioPanel = scenarioPanel;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.command.I_Command#execute()
	 */
	public void execute() {
		SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() {
			public Void doInBackground() {
				scenarioPanel.getSpaceNetFrame().getStatusBar().setStatusMessage("Filtering Network...");
				scenarioPanel.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				
				List<Location> forAddition = new ArrayList<Location>();
				List<Location> forRemoval = new ArrayList<Location>();
				for(Node node : scenarioPanel.getScenario().getDataSource().getNodeLibrary()) {
					if(!scenarioPanel.getScenario().getNetwork().getNodes().contains(node)
							&& scenarioPanel.getScenario().getScenarioType().isNodeVisible(node)) {
						forAddition.add(node);
					} else if(scenarioPanel.getScenario().getNetwork().getNodes().contains(node)
							&& !scenarioPanel.getScenario().isNodeUsed(node)
							&& !scenarioPanel.getScenario().getScenarioType().isNodeVisible(node)) {
						forRemoval.add(node);
					}
				}
				for(Edge edge : scenarioPanel.getScenario().getDataSource().getEdgeLibrary()) {
					if(!scenarioPanel.getScenario().getNetwork().getEdges().contains(edge)
							&& scenarioPanel.getScenario().getScenarioType().isEdgeVisible(edge)) {
						forAddition.add(edge);
					} else if(scenarioPanel.getScenario().getNetwork().getEdges().contains(edge)
							&& !scenarioPanel.getScenario().isEdgeUsed(edge)
							&& !scenarioPanel.getScenario().getScenarioType().isEdgeVisible(edge)) {
						forRemoval.add(edge);
					}
				}
				for(Location location : forAddition) {
					scenarioPanel.getScenario().getNetwork().add(location);
				}
				for(Location location : forRemoval) {
					scenarioPanel.getScenario().getNetwork().remove(location);
				}
				return null;
			}
			public void done() {
				SpaceNetFrame.getSingleton().getStatusBar().clearStatusMessage();
				scenarioPanel.setCursor(Cursor.getDefaultCursor());
				scenarioPanel.updateView();
			}
		};
		worker.execute();
		
	}
}
