/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.data;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import edu.mit.spacenet.domain.element.ElementType;
import edu.mit.spacenet.domain.network.edge.Edge;
import edu.mit.spacenet.domain.network.edge.EdgeType;
import edu.mit.spacenet.domain.network.node.Node;
import edu.mit.spacenet.domain.network.node.NodeType;
import edu.mit.spacenet.domain.resource.I_Resource;
import edu.mit.spacenet.domain.resource.ResourceType;
import edu.mit.spacenet.gui.ScenarioPanel;
import edu.mit.spacenet.gui.command.ChangeDataSourceCommand;
import edu.mit.spacenet.gui.command.LoadDataSourceCommand;
import edu.mit.spacenet.scenario.data.DataSourceType;
import edu.mit.spacenet.scenario.data.ElementPreview;
import edu.mit.spacenet.scenario.data.I_DataSource;

/**
 * Dialog for editing loading data sources.
 * 
 * @author Paul Grogan
 */
public class DataSourceDialog extends JDialog {
	private static final long serialVersionUID = -3048824253945313736L;
	
	private ScenarioPanel scenarioPanel;
	private I_DataSource dataSource;
	
	private JComboBox dataSourceTypeCombo;
	private AbstractDataSourcePanel dataSourcePanel;
	
	private JCheckBox nodesCheck, edgesCheck, resourcesCheck, elementsCheck;
	private JButton loadButton;
	private DefaultTableModel nodeModel, edgeModel, resourceModel, elementModel;
	private JTable nodeTable, edgeTable, resourceTable, elementTable;
	
	/**
	 * The constructor.
	 * 
	 * @param scenarioPanel the scenario panel component
	 */
	public DataSourceDialog(ScenarioPanel scenarioPanel) {
		super(scenarioPanel.getSpaceNetFrame(), "Scenario Data Source");
		this.scenarioPanel = scenarioPanel;
		this.dataSource = scenarioPanel.getScenario().getDataSource();
		
		buildDialog();
	}
	private void buildDialog() {
		JPanel contentPanel = new JPanel();
		contentPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		contentPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		contentPanel.add(new JLabel("Type: "), c);
		c.gridx++;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		dataSourceTypeCombo = new JComboBox();
		for(DataSourceType t : DataSourceType.values()) {
			dataSourceTypeCombo.addItem(t);
		}
		dataSourceTypeCombo.setRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = -2255885956722142642L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				setIcon(((DataSourceType)value).getIcon());
				return this;
			}
		});
		dataSourceTypeCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED
						&& (dataSource==null&&e.getItem()!=DataSourceType.NONE
								|| dataSource.getDataSourceType()!=e.getItem())) {
						ChangeDataSourceCommand command = new ChangeDataSourceCommand(getThis(), (DataSourceType)e.getItem());
						command.execute();
				}
			}
		});
		contentPanel.add(dataSourceTypeCombo, c);
		
		c.gridy++;
		c.weightx = 0;
		c.gridx = 0;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		dataSourcePanel = DataSourcePanelFactory.createDataSourcePanel(this, dataSource);
		contentPanel.add(dataSourcePanel, c);
		
		c.gridy++;
		c.gridx = 0;
		c.gridwidth = 1;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.FIRST_LINE_END;
		contentPanel.add(new JLabel("Update: "), c);
		c.gridx++;
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.LINE_START;
		JPanel checkPanel = new JPanel(new GridLayout(2,2));
		nodesCheck = new JCheckBox("Nodes", true);
		checkPanel.add(nodesCheck);
		edgesCheck = new JCheckBox("Edges", true);
		checkPanel.add(edgesCheck);
		resourcesCheck = new JCheckBox("Resources", true);
		checkPanel.add(resourcesCheck);
		elementsCheck = new JCheckBox("Instantiated Elements", false);
		checkPanel.add(elementsCheck);
		contentPanel.add(checkPanel, c);
		
		c.gridy++;
		c.fill = GridBagConstraints.NONE;
		loadButton = new JButton("Load", new ImageIcon(getClass().getClassLoader().getResource("icons/database_refresh.png")));
		loadButton.setMargin(new Insets(3,3,3,3));
		loadButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				loadDataSource();
			}
		});
		contentPanel.add(loadButton, c);
		
		c.gridy++;
		c.gridx = 0;
		c.anchor = GridBagConstraints.FIRST_LINE_END;
		contentPanel.add(new JLabel("Data: "), c);
		c.gridx++;
		c.weighty = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		JTabbedPane libraryTabs = new JTabbedPane();
		contentPanel.add(libraryTabs, c);
		
		nodeModel = new DefaultTableModel() {
			private static final long serialVersionUID = 1786100183756492614L;
			public Class<?> getColumnClass(int col) {
				if(col==0) return Integer.class;
				else return Object.class;
			}
			public boolean isCellEditable(int row, int col) {
				return false;
			}
		};
		nodeModel.addColumn("ID");
		nodeModel.addColumn("Type");
		nodeModel.addColumn("Name");
		nodeTable = new JTable(nodeModel);
		nodeTable.getColumnModel().getColumn(1).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if(value instanceof NodeType)
                	setIcon(((NodeType)value).getIcon());
                return this;
			}
		});
		nodeTable.setShowHorizontalLines(false);
		nodeTable.setShowVerticalLines(false);
		JScrollPane nodeScroll = new JScrollPane(nodeTable);
		nodeScroll.setBackground(Color.WHITE);
		nodeScroll.getViewport().setBackground(Color.WHITE);
		nodeScroll.setPreferredSize(new Dimension(400,200));
		libraryTabs.add(nodeScroll, "Nodes");
		
		edgeModel = new DefaultTableModel() {
			private static final long serialVersionUID = 1786100183756492614L;
			public Class<?> getColumnClass(int col) {
				if(col==0) return Integer.class;
				else return Object.class;
			}
			public boolean isCellEditable(int row, int col) {
				return false;
			}
		};
		edgeModel.addColumn("ID");
		edgeModel.addColumn("Type");
		edgeModel.addColumn("Name");
		edgeModel.addColumn("Origin");
		edgeModel.addColumn("Destination");
		edgeTable = new JTable(edgeModel);
		edgeTable.getColumnModel().getColumn(1).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if(value instanceof EdgeType)
                	setIcon(((EdgeType)value).getIcon());
                return this;
			}
		});
		edgeTable.setShowHorizontalLines(false);
		edgeTable.setShowVerticalLines(false);
		JScrollPane edgeScroll = new JScrollPane(edgeTable);
		edgeScroll.setBackground(Color.WHITE);
		edgeScroll.getViewport().setBackground(Color.WHITE);
		edgeScroll.setPreferredSize(new Dimension(400,200));
		libraryTabs.add(edgeScroll, "Edges");
		
		resourceModel = new DefaultTableModel() {
			private static final long serialVersionUID = 1786100183756492614L;
			public Class<?> getColumnClass(int col) {
				if(col==0) return Integer.class;
				else return Object.class;
			}
			public boolean isCellEditable(int row, int col) {
				return false;
			}
		};
		resourceModel.addColumn("ID");
		resourceModel.addColumn("Type");
		resourceModel.addColumn("Name");
		resourceModel.addColumn("Class of Supply");
		resourceTable = new JTable(resourceModel);
		resourceTable.getColumnModel().getColumn(1).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if(value instanceof ResourceType)
                	setIcon(((ResourceType)value).getIcon());
                return this;
			}
		});
		resourceTable.setShowHorizontalLines(false);
		resourceTable.setShowVerticalLines(false);
		JScrollPane resourceScroll = new JScrollPane(resourceTable);
		resourceScroll.setBackground(Color.WHITE);
		resourceScroll.getViewport().setBackground(Color.WHITE);
		resourceScroll.setPreferredSize(new Dimension(400,200));
		libraryTabs.add(resourceScroll, "Resources");
		
		elementModel = new DefaultTableModel() {
			private static final long serialVersionUID = 1786100183756492614L;
			public Class<?> getColumnClass(int col) {
				if(col==0) return Integer.class;
				else return Object.class;
			}
			public boolean isCellEditable(int row, int col) {
				return false;
			}
		};
		elementModel.addColumn("ID");
		elementModel.addColumn("Type");
		elementModel.addColumn("Name");
		elementTable = new JTable(elementModel);
		elementTable.getColumnModel().getColumn(1).setCellRenderer(new DefaultTableCellRenderer() {
			private static final long serialVersionUID = 1L;
			@Override
			public Component getTableCellRendererComponent(JTable table, Object value,
					boolean isSelected, boolean hasFocus, int row, int column) {
                super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
                if(value instanceof ElementType)
                	setIcon(((ElementType)value).getIcon());
                return this;
			}
		});
		elementTable.setShowHorizontalLines(false);
		elementTable.setShowVerticalLines(false);
		JScrollPane elementScroll = new JScrollPane(elementTable);
		elementScroll.setBackground(Color.WHITE);
		elementScroll.getViewport().setBackground(Color.WHITE);
		elementScroll.setPreferredSize(new Dimension(400,200));
		libraryTabs.add(elementScroll, "Elements");
		
		c.gridy++;
		c.gridwidth = 2;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LAST_LINE_END;
		c.fill = GridBagConstraints.NONE;
		JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 3, 0));
		JButton okButton = new JButton("OK", new ImageIcon(getClass().getClassLoader().getResource("icons/database_go.png")));
		okButton.setMargin(new Insets(3,3,3,3));
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dataSourcePanel.saveData();
				scenarioPanel.updateView();
				dispose();
			}
		});
		buttonPanel.add(okButton);
		JButton cancelButton = new JButton("Cancel", new ImageIcon(getClass().getClassLoader().getResource("icons/database.png")));
		cancelButton.setMargin(new Insets(3,3,3,3));
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		buttonPanel.add(cancelButton);
		contentPanel.add(buttonPanel, c);
		
		setContentPane(contentPanel);
		setModal(true);
		
		updateView();
	}
	
	/**
	 * Requests that the dialog update its fields based on the model and also
	 * requests the data source panel to update itself.
	 */
	public void updateView() {
		dataSourceTypeCombo.setSelectedItem(dataSource==null?DataSourceType.NONE:dataSource.getDataSourceType());
		
		nodesCheck.setEnabled(dataSourcePanel.canLoad());
		edgesCheck.setEnabled(dataSourcePanel.canLoad());
		resourcesCheck.setEnabled(dataSourcePanel.canLoad());
		elementsCheck.setEnabled(dataSourcePanel.canLoad());
		loadButton.setEnabled(dataSourcePanel.canLoad());
		
		while(nodeModel.getRowCount() > 0) nodeModel.removeRow(0);
		while(edgeModel.getRowCount() > 0) edgeModel.removeRow(0);
		while(resourceModel.getRowCount() > 0) resourceModel.removeRow(0);
		while(elementModel.getRowCount() > 0) elementModel.removeRow(0);
		if(dataSource!=null) {
			for(Node node : dataSource.getNodeLibrary()) {
				nodeModel.addRow(new Object[]{new Integer(node.getTid()), node.getNodeType(), node.getName()});
			}
			for(Edge edge : dataSource.getEdgeLibrary()) {
				edgeModel.addRow(new Object[]{new Integer(edge.getTid()), edge.getEdgeType(), edge.getName(), edge.getOrigin(), edge.getDestination()});
			}
			for(I_Resource r : dataSource.getResourceLibrary()) {
				resourceModel.addRow(new Object[]{new Integer(r.getTid()), r.getResourceType(), r.getName(), r.getClassOfSupply()});
			}
			for(ElementPreview e : dataSource.getElementPreviewLibrary()) {
				elementModel.addRow(new Object[]{new Integer(e.ID), e.TYPE, e.NAME});
			}
		}
		nodeTable.repaint();
		edgeTable.repaint();
		resourceTable.repaint();
		elementTable.repaint();
	}
	private DataSourceDialog getThis() {
		return this;
	}
	
	/**
	 * Gets the scenario panel.
	 * 
	 * @return the scenario panel
	 */
	public ScenarioPanel getScenarioPanel() {
		return scenarioPanel;
	}
	
	/**
	 * Gets the data source.
	 * 
	 * @return the data source
	 */
	public I_DataSource getDataSource() {
		return dataSource;
	}
	
	/**
	 * Loads the data source.
	 */
	public void loadDataSource() {
		dataSourcePanel.saveData();
		
		LoadDataSourceCommand command = new LoadDataSourceCommand(this);
		command.execute();
	}
	
	/**
	 * Gets whether the already-instantiated elements should be updated.
	 * 
	 * @return whether the instantiated elements should be updated
	 */
	public boolean updateInstantiatedElements() {
		return elementsCheck.isSelected();
	}
	
	/**
	 * Update nodes.
	 * 
	 * @return true, if existing nodes are to be updated
	 */
	public boolean updateNodes() {
		return nodesCheck.isSelected();
	}
	
	/**
	 * Update edges.
	 * 
	 * @return true, if existing edges are to be updated
	 */
	public boolean updateEdges() {
		return edgesCheck.isSelected();
	}
	
	/**
	 * Update resources.
	 * 
	 * @return true, if existing resources are to be updated
	 */
	public boolean updateResources() {
		return resourcesCheck.isSelected();
	}
	
	/**
	 * Builds and displays the data source dialog.
	 * 
	 * @param scenarioPanel the scenario panel component
	 */
	public static void createAndShowGUI(ScenarioPanel scenarioPanel) {
		DataSourceDialog dialog = new DataSourceDialog(scenarioPanel);
		dialog.pack();
		dialog.setLocationRelativeTo(null);
		dialog.setVisible(true);
	}
}
