/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.element;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import edu.mit.spacenet.domain.ClassOfSupply;
import edu.mit.spacenet.domain.Environment;
import edu.mit.spacenet.domain.element.ElementIcon;
import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.element.I_State;
import edu.mit.spacenet.domain.element.PartApplication;
import edu.mit.spacenet.domain.element.State;
import edu.mit.spacenet.domain.resource.ResourceType;
import edu.mit.spacenet.gui.component.UnitsWrapper;
import edu.mit.spacenet.gui.event.EventDialog;

/**
 * Component for viewing and editing elements.
 * 
 * @author Paul Grogan
 */
public class ElementDialog extends JDialog {
	private static final long serialVersionUID = 5400688589831319535L;

	private I_Element element;
	private EventDialog eventDialog;
	
	private JTextField nameText;
	private JComboBox iconCombo, classOfSupplyCombo, environmentCombo;
	private SpinnerNumberModel massModel, volumeModel;
	private JSpinner massSpinner, volumeSpinner;
	private I_State initialState;
	private DefaultListModel statesModel, partsModel;
	private JList statesList, partsList;
	private JButton editStateButton, setInitialStateButton, removeStateButton, 
		editPartButton, removePartButton;
	private List<AbstractElementPanel> elementPanels;
	private JButton okButton;
	
	/**
	 * The constructor.
	 * 
	 * @param dialog the event dialog
	 * @param element the element
	 */
	public ElementDialog(EventDialog dialog, I_Element element) {
		super(dialog, element.getElementType().getName());
		this.eventDialog = dialog;
		this.element = element;
		buildDialog();
		initialize();
	}
	private void buildDialog() {
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		JPanel contentPanel = new JPanel();
		contentPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		contentPanel.setLayout(new GridBagLayout());
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.LINE_END;
		contentPanel.add(new JLabel("Name: "), c);
		c.gridy++;
		contentPanel.add(new JLabel("Icon: "), c);
		c.gridy++;
		contentPanel.add(new JLabel("Class of Supply: "), c);
		c.gridy++;
		contentPanel.add(new JLabel("Environment: "), c);
		c.gridy++;
		contentPanel.add(new JLabel("Mass: "), c);
		c.gridx+=2;
		c.weightx = 1;
		contentPanel.add(new JLabel("Volume: "), c);
		c.weightx = 0;
		c.gridx-=2;
		c.gridy++;
		c.anchor = GridBagConstraints.FIRST_LINE_END;
		contentPanel.add(new JLabel("States: "), c);
		c.gridy+=2;
		c.anchor = GridBagConstraints.FIRST_LINE_END;
		contentPanel.add(new JLabel("Parts: "), c);
		
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 0;
		c.gridwidth = 3;
		nameText = new JTextField(20);
		contentPanel.add(nameText, c);
		c.gridy++;
		c.fill = GridBagConstraints.NONE;
		c.gridwidth = 3;
		JPanel iconPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		iconCombo = new JComboBox();
		for(ElementIcon i : ElementIcon.values()) {
			iconCombo.addItem(i.getIcon());
		}
		iconCombo.setRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = -2255885956722142642L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				setIcon((ImageIcon)value);
				setText(ElementIcon.getInstance((ImageIcon)value).getName());
				return this;
			}
		});
		iconPanel.add(iconCombo);
		JPanel dividerPanel = new JPanel();
		dividerPanel.setPreferredSize(new Dimension(5,1));
		iconPanel.add(dividerPanel);
		JButton resetIconButton = new JButton("Reset");
		resetIconButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				iconCombo.setSelectedItem(element.getElementType().getIcon());
			}
		});
		iconPanel.add(resetIconButton);
		contentPanel.add(iconPanel, c);
		c.gridy++;
		c.gridwidth = 1;
		c.weightx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 3;
		classOfSupplyCombo = new JComboBox();
		for(ClassOfSupply cos : ClassOfSupply.values())
			classOfSupplyCombo.addItem(cos);
		contentPanel.add(classOfSupplyCombo, c);
		c.gridy++;
		c.gridwidth = 1;
		environmentCombo = new JComboBox();
		for(Environment e : Environment.values())
			environmentCombo.addItem(e);
		contentPanel.add(environmentCombo, c);
		c.gridy++;
		c.gridx = 1;
		massModel = new SpinnerNumberModel(0,0,10000,0.5);
		massSpinner = new JSpinner(massModel);
		contentPanel.add(new UnitsWrapper(massSpinner, "kg"), c);
		c.gridx+=2;
		c.weightx = 0;
		volumeModel = new SpinnerNumberModel(0,0,1000,0.001);
		volumeSpinner = new JSpinner(volumeModel);
		contentPanel.add(new UnitsWrapper(volumeSpinner, "m^3"), c);
		c.gridx-=2;
		c.gridwidth = 3;
		c.gridy++;
		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1;
		statesModel = new DefaultListModel();
		statesList = new JList(statesModel);
		statesList.setCellRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = 1271331296677711150L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				JLabel label = (JLabel)super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				label.setIcon((((I_State)value).getStateType()).getIcon());
				if(value.equals(initialState)) {
					label.setFont(label.getFont().deriveFont(Font.BOLD));
				} else {
					label.setFont(label.getFont().deriveFont(Font.PLAIN));
				}
				return label;
			}
		});
		statesList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if(e.getClickCount()==2 && statesList.getSelectedIndex()>=0) {
					StateDialog.createAndShowGUI(getThis(), (I_State)statesList.getSelectedValue());
				}
			}
		});
		statesList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if(!statesList.isSelectionEmpty()) {
					partsList.clearSelection();
				}
				if(statesList.getSelectedIndices().length == 1) {
					editStateButton.setEnabled(true);
					setInitialStateButton.setEnabled(true);
					removeStateButton.setEnabled(true);
				} else if(statesList.getSelectedIndices().length > 1) {
					editStateButton.setEnabled(false);
					setInitialStateButton.setEnabled(false);
					removeStateButton.setEnabled(true);
				} else {
					editStateButton.setEnabled(false);
					setInitialStateButton.setEnabled(false);
					removeStateButton.setEnabled(false);
				}	
			}
		});
		JScrollPane statesScroll = new JScrollPane(statesList);
		statesScroll.setPreferredSize(new Dimension(250,50));
		contentPanel.add(statesScroll, c);
		c.gridy++;
		c.fill = GridBagConstraints.NONE;
		c.weighty = 0;
		JPanel statesButtonPanel = new JPanel();
		statesButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 3, 0));
		JButton addStateButton = new JButton("Add", new ImageIcon(getClass().getClassLoader().getResource("icons/clock_add.png")));
		addStateButton.setMargin(new Insets(3,3,3,3));
		addStateButton.setToolTipText("Add State");
		addStateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				StateDialog.createAndShowGUI(getThis(), new State());
			}
		});
		statesButtonPanel.add(addStateButton);
		editStateButton = new JButton("Edit", new ImageIcon(getClass().getClassLoader().getResource("icons/clock_edit.png")));
		editStateButton.setMargin(new Insets(3,3,3,3));
		editStateButton.setToolTipText("Edit State");
		editStateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				StateDialog.createAndShowGUI(getThis(), (I_State)statesList.getSelectedValue());
			}
		});
		editStateButton.setEnabled(false);
		statesButtonPanel.add(editStateButton);
		setInitialStateButton = new JButton("Set Initial", new ImageIcon(getClass().getClassLoader().getResource("icons/clock.png")));
		setInitialStateButton.setMargin(new Insets(3,3,3,3));
		setInitialStateButton.setToolTipText("Set Initial");
		setInitialStateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				initialState = (I_State)statesList.getSelectedValue();
				statesList.repaint();
			}
		});
		setInitialStateButton.setEnabled(false);
		statesButtonPanel.add(setInitialStateButton);
		removeStateButton = new JButton("Remove", new ImageIcon(getClass().getClassLoader().getResource("icons/clock_delete.png")));
		removeStateButton.setMargin(new Insets(3,3,3,3));
		removeStateButton.setToolTipText("Remove State");
		removeStateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for(Object state : statesList.getSelectedValues()) {
					statesModel.removeElement(state);
					if(state.equals(initialState)) {
						if(statesModel.size()>0) {
							initialState = (I_State)statesModel.toArray()[0];
						} else {
							initialState = null;
						}
					}
				}				
				statesList.repaint();
			}
		});
		removeStateButton.setEnabled(false);
		statesButtonPanel.add(removeStateButton);
		contentPanel.add(statesButtonPanel, c);
		c.gridy++;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1;
		partsModel = new DefaultListModel();
		partsList = new JList(partsModel);
		partsList.setCellRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = 1271331296677711150L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				JLabel label = (JLabel)super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				label.setIcon(ResourceType.ITEM.getIcon());
				return label;
			}
		});
		partsList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if(e.getClickCount()==2 && partsList.getSelectedIndex()>=0) {
					PartApplicationDialog.createAndShowGUI(getThis(), (PartApplication)partsList.getSelectedValue());
				}
			}
		});
		partsList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if(!partsList.isSelectionEmpty()) {
					statesList.clearSelection();
				}
				if(partsList.getSelectedIndices().length == 1) {
					editPartButton.setEnabled(true);
					removePartButton.setEnabled(true);
				} else if(statesList.getSelectedIndices().length > 1) {
					editPartButton.setEnabled(false);
					removePartButton.setEnabled(true);
				} else {
					editPartButton.setEnabled(false);
					removePartButton.setEnabled(false);
				}
			}
		});
		JScrollPane partsScroll = new JScrollPane(partsList);
		partsScroll.setPreferredSize(new Dimension(250,50));
		contentPanel.add(partsScroll, c);
		c.gridy++;
		c.fill = GridBagConstraints.NONE;
		c.weighty = 0;
		JPanel partsButtonPanel = new JPanel();
		partsButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 3, 0));
		JButton addPartButton = new JButton("Add", new ImageIcon(getClass().getClassLoader().getResource("icons/wrench_add.png")));
		addPartButton.setMargin(new Insets(3,3,3,3));
		addPartButton.setToolTipText("Add Part");
		addPartButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				PartApplicationDialog.createAndShowGUI(getThis(), new PartApplication());
			}
		});
		partsButtonPanel.add(addPartButton);
		editPartButton = new JButton("Edit", new ImageIcon(getClass().getClassLoader().getResource("icons/wrench_edit.png")));
		editPartButton.setMargin(new Insets(3,3,3,3));
		editPartButton.setToolTipText("Edit Part");
		editPartButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				PartApplicationDialog.createAndShowGUI(getThis(), (PartApplication)partsList.getSelectedValue());
			}
		});
		editPartButton.setEnabled(false);
		partsButtonPanel.add(editPartButton);
		removePartButton = new JButton("Remove", new ImageIcon(getClass().getClassLoader().getResource("icons/wrench_delete.png")));
		removePartButton.setMargin(new Insets(3,3,3,3));
		removePartButton.setToolTipText("Remove Part");
		removePartButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				for(Object part : partsList.getSelectedValues()) {
					partsModel.removeElement(part);
				}
			}
		});
		removePartButton.setEnabled(false);
		partsButtonPanel.add(removePartButton);
		contentPanel.add(partsButtonPanel, c);
		
		c.gridy++;
		c.gridx = 0;
		c.gridwidth = 4;
		c.weightx = 1;
		c.weighty = 1;
		c.anchor = GridBagConstraints.CENTER;
		c.fill = GridBagConstraints.BOTH;
		elementPanels = ElementPanelFactory.createElementPanel(this, element);
		for(AbstractElementPanel panel : elementPanels) {
			contentPanel.add(panel, c);
			c.gridy++;
		}
		
		c.gridy++;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LAST_LINE_END;
		c.fill = GridBagConstraints.NONE;
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 3, 0));
		contentPanel.add(buttonPanel, c);
		okButton = new JButton("OK", new ImageIcon(getClass().getClassLoader().getResource("icons/brick_go.png")));
		okButton.setMargin(new Insets(3,3,3,3));
		okButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveElement();
				eventDialog.updateView();
				dispose();
			}
		});
		buttonPanel.add(okButton);
		JButton cancelButton = new JButton("Cancel", new ImageIcon(getClass().getClassLoader().getResource("icons/brick_delete.png")));
		cancelButton.setMargin(new Insets(3,3,3,3));
		cancelButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				dispose();
			}
		});
		buttonPanel.add(cancelButton);

		getRootPane().setDefaultButton(okButton);
		setContentPane(contentPanel);
		setMinimumSize(new Dimension(300,150));
		setModal(true);
	}
	private void initialize() {
		nameText.setText(element.getName());
		iconCombo.setSelectedItem(element.getIcon());
		if(element.getClassOfSupply()!=null) 
			classOfSupplyCombo.setSelectedItem(element.getClassOfSupply());
		environmentCombo.setSelectedItem(element.getEnvironment());
		massModel.setValue(element.getMass());
		volumeModel.setValue(element.getVolume());
		for(I_State state : element.getStates()) {
			statesModel.addElement(state);
		}
		initialState = element.getCurrentState();
		for(PartApplication part : element.getParts()) {
			partsModel.addElement(part);
		}
	}
	private void saveElement() {
		element.setName(nameText.getText());
		if(element.getElementType().getIcon().equals(iconCombo.getSelectedItem())) {
			element.setIcon(null);
		} else element.setIcon((ImageIcon)iconCombo.getSelectedItem());
		element.setClassOfSupply((ClassOfSupply)classOfSupplyCombo.getSelectedItem());
		element.setEnvironment((Environment)environmentCombo.getSelectedItem());
		element.setMass(massModel.getNumber().doubleValue());
		element.setVolume(volumeModel.getNumber().doubleValue());
		element.getStates().clear();
		for(Object state : statesModel.toArray()) {
			element.getStates().add((I_State)state);
		}
		element.setCurrentState(initialState);
		element.getParts().clear();
		for(Object part : partsModel.toArray()) {
			element.getParts().add((PartApplication)part);
		}
		for(AbstractElementPanel panel : elementPanels) {
			panel.saveElement();
		}
	}
	private ElementDialog getThis() {
		return this;
	}
	
	/**
	 * Gets the element.
	 * 
	 * @return the element
	 */
	public I_Element getElement() {
		return element;
	}
	
	/**
	 * Gets the event dialog.
	 * 
	 * @return the event dialog
	 */
	public EventDialog getCreateEventDialog() {
		return eventDialog;
	}
	
	/**
	 * Gets whether the states model contains a state.
	 * 
	 * @param state the state
	 * 
	 * @return whether the model contains the state
	 */
	public boolean containsState(I_State state) {
		return statesModel.contains(state);
	}
	
	/**
	 * Adds a state to the states model.
	 * 
	 * @param state the state
	 */
	public void addState(I_State state) {
		if(initialState==null) initialState = state;
		statesModel.addElement(state);
	}
	
	/**
	 * Gets whether the parts application model contains a part.
	 * 
	 * @param part the part
	 * 
	 * @return whether the model contains the part
	 */
	public boolean containsPart(PartApplication part) {
		return partsModel.contains(part);
	}
	
	/**
	 * Adds a part to the parts application model.
	 * 
	 * @param part the part
	 */
	public void addPart(PartApplication part) {
		partsModel.addElement(part);
	}
	
	/**
	 * Request that the dialog update itself.
	 */
	public void updateView() {
		partsList.repaint();
		statesList.repaint();
	}
	
	/**
	 * Creates and shows the dialog.
	 * 
	 * @param eventDialog the parent event dialog component
	 * @param element the element
	 */
	public static void createAndShowGUI(EventDialog eventDialog, I_Element element) {
		ElementDialog dialog = new ElementDialog(eventDialog, element);
		dialog.pack();
		dialog.setLocationRelativeTo(null);
		dialog.setVisible(true);
	}
}
