/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.element;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import edu.mit.spacenet.domain.element.I_State;
import edu.mit.spacenet.domain.element.StateType;
import edu.mit.spacenet.domain.model.DemandModelFactory;
import edu.mit.spacenet.domain.model.DemandModelType;
import edu.mit.spacenet.domain.model.I_DemandModel;
import edu.mit.spacenet.gui.component.DropDownButton;
import edu.mit.spacenet.gui.model.DemandModelDialog;

/**
 * A dialog for viewing and editing element operational states.
 * 
 * @author Paul Grogan
 */
public class StateDialog extends JDialog {
	private static final long serialVersionUID = -2572640622351398778L;
	
	private JTextField txtName;
	private JComboBox ddlType;
	private SpinnerNumberModel dutyCycleModel;
	private JSpinner dutyCycleSpinner;
	private DefaultListModel demandModelsModel;
	private JList demandModelsList;
	private JScrollPane demandModelsScroll;
	private DropDownButton btnAddDemandModel;
	private JMenuItem addImpulseDemandModel, addTimedImpulseDemandModel, addRatedDemandModel, 
		addBoilOffDemandModel, addSparingByMassDemandModel, addIsruProductionModel;
	private JButton btnEditDemandModel, btnRemoveDemandModel, btnOk, btnCancel;
	private JPanel demandModelButtonPanel, buttonPanel;
	
	private I_State state;
	private ElementDialog elementDialog;
	
	/**
	 * Instantiates a new state dialog.
	 * 
	 * @param elementDialog the element dialog
	 * @param state the state
	 */
	public StateDialog(ElementDialog elementDialog, I_State state) {
		super(elementDialog, "Operational State");
		this.elementDialog = elementDialog;
		this.state = state;
		
		buildDialog();
		
		initialize();
	}
	private void buildDialog() {
		JPanel contentPanel = new JPanel();
		contentPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		contentPanel.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.LINE_END;
		
		contentPanel.add(new JLabel("Name"), c);
		c.gridy++;
		contentPanel.add(new JLabel("Type: "), c);
		c.gridy++;
		contentPanel.add(new JLabel("Duty Cycle: "), c);
		c.gridy++;
		c.anchor = GridBagConstraints.FIRST_LINE_END;
		contentPanel.add(new JLabel("Demand Models: "), c);
		
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridy = 0;
		c.gridx = 1;
		c.weightx = 1;
		txtName = new JTextField(25);
		contentPanel.add(txtName, c);
		c.gridy++;
		c.fill = GridBagConstraints.NONE;
		ddlType = new JComboBox();
		for(StateType t : StateType.values()) 
			ddlType.addItem(t);
		ddlType.setRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = -2255885956722142642L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				setIcon(((StateType)value).getIcon());
				return this;
			}
		});
		contentPanel.add(ddlType, c);
		c.gridy++;
		dutyCycleModel = new SpinnerNumberModel(1.0, 0.0, 1.0, 0.1);
		dutyCycleSpinner = new JSpinner(dutyCycleModel);
		dutyCycleSpinner.setPreferredSize(new Dimension(50,20));
		contentPanel.add(dutyCycleSpinner, c);
		
		c.gridy++;
		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1;
		demandModelsModel = new DefaultListModel();
		demandModelsList = new JList(demandModelsModel);
		demandModelsList.setCellRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = 1271331296677711150L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				JLabel label = (JLabel)super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				label.setIcon(((I_DemandModel)value).getDemandModelType().getIcon());
				return label;
			}
		});
		demandModelsList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if(e.getClickCount()==2 && demandModelsList.getSelectedIndex()>=0) {
					DemandModelDialog.createAndShowGUI(getThis(), (I_DemandModel)demandModelsList.getSelectedValue());
				}
			}
		});
		demandModelsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		demandModelsList.setVisibleRowCount(3);
		demandModelsList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				if(demandModelsList.getSelectedIndex() >= 0) {
					btnEditDemandModel.setEnabled(true);
					btnRemoveDemandModel.setEnabled(true);
				} else {
					btnEditDemandModel.setEnabled(false);
					btnRemoveDemandModel.setEnabled(false);
				}
			}
		});
		demandModelsScroll = new JScrollPane(demandModelsList);
		contentPanel.add(demandModelsScroll, c);
		
		c.gridy++;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		demandModelButtonPanel = new JPanel();
		demandModelButtonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 3, 0));
		contentPanel.add(demandModelButtonPanel, c);
		
		btnAddDemandModel = new DropDownButton("Add", new ImageIcon(getClass().getClassLoader().getResource("icons/comment_add.png")));
		btnAddDemandModel.getButton().setMargin(new Insets(3,3,3,3));
		btnAddDemandModel.setToolTipText("Add Demand Model");
		demandModelButtonPanel.add(btnAddDemandModel);
		
		addImpulseDemandModel = new JMenuItem(DemandModelType.IMPULSE.getName(), DemandModelType.IMPULSE.getIcon());
		addImpulseDemandModel.setToolTipText("Add a one-time demand for resources");
		addImpulseDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DemandModelDialog.createAndShowGUI(getThis(), 
						DemandModelFactory.createDemandModel(getElementDialog().getElement(), DemandModelType.IMPULSE));
			}
		});
		btnAddDemandModel.getMenu().add(addImpulseDemandModel);
		addTimedImpulseDemandModel = new JMenuItem(DemandModelType.TIMED_IMPULSE.getName(), DemandModelType.TIMED_IMPULSE.getIcon());
		addTimedImpulseDemandModel.setToolTipText("Add a one-time demand for resources at a given time");
		addTimedImpulseDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DemandModelDialog.createAndShowGUI(getThis(), 
						DemandModelFactory.createDemandModel(getElementDialog().getElement(), DemandModelType.TIMED_IMPULSE));
			}
		});
		btnAddDemandModel.getMenu().add(addTimedImpulseDemandModel);
		addRatedDemandModel = new JMenuItem(DemandModelType.RATED.getName(), DemandModelType.RATED.getIcon());
		addRatedDemandModel.setToolTipText("Add a recurring demand for resources based on a demand rate");
		addRatedDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DemandModelDialog.createAndShowGUI(getThis(),
						DemandModelFactory.createDemandModel(getElementDialog().getElement(), DemandModelType.RATED));
			}
		});
		btnAddDemandModel.getMenu().add(addRatedDemandModel);
		addBoilOffDemandModel = new JMenuItem(DemandModelType.BOIL_OFF.getName(), DemandModelType.BOIL_OFF.getIcon());
		addBoilOffDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DemandModelDialog.createAndShowGUI(getThis(),
						DemandModelFactory.createDemandModel(getElementDialog().getElement(), DemandModelType.BOIL_OFF));
			}
		});
		btnAddDemandModel.getMenu().add(addBoilOffDemandModel);
		addSparingByMassDemandModel = new JMenuItem(DemandModelType.SPARING_BY_MASS.getName(), DemandModelType.SPARING_BY_MASS.getIcon());
		addSparingByMassDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DemandModelDialog.createAndShowGUI(getThis(),
						DemandModelFactory.createDemandModel(getElementDialog().getElement(), DemandModelType.SPARING_BY_MASS));
			}
		});
		btnAddDemandModel.getMenu().add(addSparingByMassDemandModel);
		addIsruProductionModel = new JMenuItem(DemandModelType.ISRU.getName(), DemandModelType.ISRU.getIcon());
		addIsruProductionModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DemandModelDialog.createAndShowGUI(getThis(),
						DemandModelFactory.createDemandModel(getElementDialog().getElement(), DemandModelType.ISRU));
			}
		});
		btnAddDemandModel.getMenu().add(addIsruProductionModel);
		demandModelButtonPanel.add(btnAddDemandModel);
		
		btnEditDemandModel = new JButton("Edit", new ImageIcon(getClass().getClassLoader().getResource("icons/comment_edit.png")));
		btnEditDemandModel.setMargin(new Insets(3,3,3,3));
		btnEditDemandModel.setToolTipText("Edit Demand Model");
		btnEditDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DemandModelDialog.createAndShowGUI(getThis(), (I_DemandModel)demandModelsList.getSelectedValue());
			}
		});
		btnEditDemandModel.setEnabled(false);
		demandModelButtonPanel.add(btnEditDemandModel);
		
		btnRemoveDemandModel = new JButton("Remove", new ImageIcon(getClass().getClassLoader().getResource("icons/comment_delete.png")));
		btnRemoveDemandModel.setMargin(new Insets(3,3,3,3));
		btnRemoveDemandModel.setToolTipText("Remove Demand Model");
		btnRemoveDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				demandModelsModel.removeElement(demandModelsList.getSelectedValue());
				updateView();
			}
		});
		btnRemoveDemandModel.setEnabled(false);
		demandModelButtonPanel.add(btnRemoveDemandModel);
		
		c.gridy++;
		c.gridx = 0;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.LINE_END;
		btnOk = new JButton("OK", new ImageIcon(getClass().getClassLoader().getResource("icons/clock_go.png")));
		btnOk.setMargin(new Insets(3,3,3,3));
		btnOk.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveState();
				if(!elementDialog.containsState(state)) {
					elementDialog.addState(state);
				}
				elementDialog.updateView();
				dispose();
			}
		});
		getRootPane().setDefaultButton(btnOk);
		btnCancel = new JButton("Cancel", new ImageIcon(getClass().getClassLoader().getResource("icons/clock_delete.png")));
		btnCancel.setMargin(new Insets(3,3,3,3));
		btnCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getThis().dispose();
			}
		});
		buttonPanel = new JPanel();
		buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 3, 0));
		buttonPanel.add(btnOk);
		buttonPanel.add(btnCancel);
		contentPanel.add(buttonPanel, c);
		
		setContentPane(contentPanel);
		setModal(true);
	}
	private void initialize() {
		txtName.setText(state.getName());
		ddlType.setSelectedItem(state.getStateType());
		dutyCycleModel.setValue(state.getDutyCycle());
		for(I_DemandModel model : state.getDemandModels()) {
			demandModelsModel.addElement(model);
		}
		updateView();
	}
	private void saveState() {
		state.setName(txtName.getText());
		state.setStateType((StateType)ddlType.getSelectedItem());
		state.setDutyCycle(dutyCycleModel.getNumber().doubleValue());
		state.getDemandModels().clear();
		for(Object o : demandModelsModel.toArray()) {
			state.getDemandModels().add((I_DemandModel)o);
		}
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		demandModelsList.repaint();
	}
	
	/**
	 * Gets the this.
	 * 
	 * @return the this
	 */
	public StateDialog getThis() {
		return this;
	}
	
	/**
	 * Gets the element dialog.
	 * 
	 * @return the element dialog
	 */
	public ElementDialog getElementDialog() {
		return elementDialog;
	}
	
	/**
	 * Gets the state.
	 * 
	 * @return the state
	 */
	public I_State getState() {
		return state;
	}
	
	/**
	 * Contains demand model.
	 * 
	 * @param demandModel the demand model
	 * 
	 * @return true, if successful
	 */
	public boolean containsDemandModel(I_DemandModel demandModel) {
		return demandModelsModel.contains(demandModel);
	}
	
	/**
	 * Adds the demand model.
	 * 
	 * @param demandModel the demand model
	 */
	public void addDemandModel(I_DemandModel demandModel) {
		demandModelsModel.addElement(demandModel);
	}
	
	/**
	 * Creates the and show gui.
	 * 
	 * @param elementDialog the element dialog
	 * @param state the state
	 */
	public static void createAndShowGUI(ElementDialog elementDialog, I_State state) {
		StateDialog dialog = new StateDialog(elementDialog, state);
		dialog.pack();
		dialog.setLocationRelativeTo(null);
		dialog.setVisible(true);
	}
}
