/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.mission;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Collections;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.toedter.calendar.JDateChooser;

import edu.mit.spacenet.domain.event.EventFactory;
import edu.mit.spacenet.domain.event.EventType;
import edu.mit.spacenet.domain.event.I_Event;
import edu.mit.spacenet.domain.event.I_Process;
import edu.mit.spacenet.domain.event.I_Transport;
import edu.mit.spacenet.domain.model.CrewConsumablesDemandModel;
import edu.mit.spacenet.domain.model.DemandModelFactory;
import edu.mit.spacenet.domain.model.DemandModelType;
import edu.mit.spacenet.domain.model.I_DemandModel;
import edu.mit.spacenet.domain.network.node.Node;
import edu.mit.spacenet.gui.SpaceNetFrame;
import edu.mit.spacenet.gui.component.ContainerComboBox;
import edu.mit.spacenet.gui.component.DropDownButton;
import edu.mit.spacenet.gui.event.EventDialog;
import edu.mit.spacenet.gui.model.DemandModelDialog;
import edu.mit.spacenet.scenario.Mission;

/**
 * A component used to display and edit a mission.
 * 
 * @author Paul Grogan
 */
public class MissionPanel extends JPanel {
	private static final long serialVersionUID = -4374168110250821161L;
	
	private MissionsTab missionsTab;
	private Mission mission;
	
	private JComboBox ddlOrigin, ddlDestination, ddlReturnOrigin, ddlReturnDestination;
	private JTextField txtName;
	private JDateChooser calStartDate;
	private DefaultListModel demandModelsModel;
	private JList demandModelsList;
	private EventTable eventsTable;
	private JButton btnEditDemandModel, btnRemoveDemandModel, btnEditEvent, btnRemoveEvent;
	
	/**
	 * Instantiates a new mission panel.
	 * 
	 * @param missionsTab the missions tab
	 */
	public MissionPanel(MissionsTab missionsTab) {
		this.missionsTab = missionsTab;
		buildPanel();
	}
	private void buildPanel() {
		setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		c.weighty = 0;
		c.weightx = 0;
		c.gridy = 0;
		c.gridx = 0;
		add(new JLabel("Name: "), c);
		c.gridy = 1;
		add(new JLabel("Start Date: "), c);
		c.gridy++;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1;
		JPanel departurePanel = new JPanel();
		departurePanel.setBorder(BorderFactory.createTitledBorder("Departure Trip"));
		departurePanel.setLayout(new GridBagLayout());
		GridBagConstraints g = new GridBagConstraints();
		g.gridx = 0;
		g.gridy = 0;
		g.insets = new Insets(2,2,2,2);
		g.anchor = GridBagConstraints.LINE_END;
		departurePanel.add(new JLabel("From: "), g);
		g.gridy++;
		departurePanel.add(new JLabel("To: "), g);
		g.gridy--;
		g.gridx++;
		g.anchor = GridBagConstraints.LINE_START;
		g.fill = GridBagConstraints.BOTH;
		g.weightx = 1;
		ddlOrigin = new ContainerComboBox();
		ddlOrigin.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED) {
					mission.setOrigin((Node)ddlOrigin.getSelectedItem());
				}
			}
		});
		departurePanel.add(ddlOrigin, g);
		g.gridy++;
		ddlDestination = new ContainerComboBox();
		ddlDestination.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				if(e.getStateChange()==ItemEvent.SELECTED) {
					mission.setDestination((Node)ddlDestination.getSelectedItem());
				}
			}
		});
		departurePanel.add(ddlDestination, g);
		JPanel returnPanel = new JPanel();
		returnPanel.setBorder(BorderFactory.createTitledBorder("Return Trip"));
		returnPanel.setLayout(new GridBagLayout());
		g = new GridBagConstraints();
		g.gridx = 0;
		g.gridy = 0;
		g.insets = new Insets(2,2,2,2);
		g.anchor = GridBagConstraints.LINE_END;
		returnPanel.add(new JLabel("From: "), g);
		g.gridy++;
		returnPanel.add(new JLabel("To: "), g);
		g.gridy--;
		g.gridx++;
		g.anchor = GridBagConstraints.LINE_START;
		g.fill = GridBagConstraints.BOTH;
		g.weightx = 1;
		ddlReturnOrigin = new ContainerComboBox();
		ddlReturnOrigin.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				mission.setReturnOrigin((Node)ddlReturnOrigin.getSelectedItem());
			}
		});
		returnPanel.add(ddlReturnOrigin, g);
		g.gridy++;
		ddlReturnDestination = new ContainerComboBox();
		ddlReturnDestination.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				mission.setReturnDestination((Node)ddlReturnDestination.getSelectedItem());
			}
		});
		returnPanel.add(ddlReturnDestination, g);

		JPanel nodePanel = new JPanel();
		nodePanel.setBorder(BorderFactory.createEmptyBorder());
		nodePanel.setLayout(new BoxLayout(nodePanel, BoxLayout.LINE_AXIS));
		nodePanel.add(departurePanel);
		nodePanel.add(returnPanel);
		add(nodePanel, c);
		
		c.gridy++;
		c.gridx = 0;
		c.weightx = 0;
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.fill = GridBagConstraints.NONE;
		add(new JLabel("Demand Models: "), c);
		c.gridy+=3;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.LINE_START;
		add(new JLabel("Events: "), c);
		
		c.gridx = 1;
		c.gridy = 0;
		c.fill = GridBagConstraints.BOTH;
		txtName = new JTextField(20);
		txtName.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent e) {
				mission.setName(txtName.getText());
				eventsTable.updateView();
			}
		});
		add(txtName, c);
		c.gridy++;
		calStartDate = new JDateChooser();
		calStartDate.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent e) {
				if(e.getPropertyName().equals("date")) {
					mission.setStartDate(calStartDate.getDate());
					updateView();
				}
			}
		});
		add(calStartDate, c);
		c.gridy+=3;
		c.gridwidth = 2;
		c.gridx = 0;
		c.weighty = 0.1;
		c.weightx = 1;
		demandModelsModel = new DefaultListModel();
		demandModelsList = new JList(demandModelsModel);
		demandModelsList.setCellRenderer(new DefaultListCellRenderer() {
			private static final long serialVersionUID = 1271331296677711150L;
			public Component getListCellRendererComponent(JList list, Object value, 
					int index, boolean isSelected, boolean cellHasFocus) {
				JLabel label = (JLabel)super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				label.setIcon(((I_DemandModel)value).getDemandModelType().getIcon());
				return label;
			}
		});
		demandModelsList.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if(e.getClickCount()==2 && demandModelsList.getSelectedIndex()>=0) {
					DemandModelDialog.createAndShowGUI(missionsTab, (I_DemandModel)demandModelsList.getSelectedValue());
				}
			}
		});
		demandModelsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		demandModelsList.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				updateButtons();
			}
		});
		JScrollPane demandModelsScroll = new JScrollPane(demandModelsList);
		demandModelsScroll.setPreferredSize(new Dimension(125,20));
		demandModelsScroll.setBackground(Color.WHITE);
		demandModelsScroll.getViewport().setBackground(Color.WHITE);
		add(demandModelsScroll, c);
		c.gridy++;
		c.gridx = 0;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		JPanel demandsButtonPanel = new JPanel();
		demandsButtonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 3, 0));
		DropDownButton btnAddDemandModel = new DropDownButton("Add Model", new ImageIcon(getClass().getClassLoader().getResource("icons/comment_add.png")));
		btnAddDemandModel.getButton().setMargin(new Insets(3,3,3,3));
		btnAddDemandModel.setToolTipText("Add Demand Model");
		
		JMenuItem addImpulseDemandModel = new JMenuItem(DemandModelType.IMPULSE.getName(), DemandModelType.IMPULSE.getIcon());
		addImpulseDemandModel.setToolTipText("Add a one-time demand for resources");
		addImpulseDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DemandModelDialog.createAndShowGUI(missionsTab, 
						DemandModelFactory.createDemandModel(mission, DemandModelType.IMPULSE));
			}
		});
		btnAddDemandModel.getMenu().add(addImpulseDemandModel);
		JMenuItem addRatedDemandModel = new JMenuItem(DemandModelType.RATED.getName(), DemandModelType.RATED.getIcon());
		addRatedDemandModel.setToolTipText("Add a recurring demand for resources based on a demand rate");
		addRatedDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DemandModelDialog.createAndShowGUI(missionsTab, 
						DemandModelFactory.createDemandModel(mission, DemandModelType.RATED));
			}
		});
		btnAddDemandModel.getMenu().add(addRatedDemandModel);
		JMenuItem addCrewConsumablesDemandModel = new JMenuItem(DemandModelType.CREW_CONSUMABLES.getName(), DemandModelType.CREW_CONSUMABLES.getIcon());
		addCrewConsumablesDemandModel.setToolTipText("Use the crew consumables model");
		addCrewConsumablesDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				CrewConsumablesDemandModel model = new CrewConsumablesDemandModel(mission);
				DemandModelDialog.createAndShowGUI(missionsTab, model);
			}
		});
		btnAddDemandModel.getMenu().add(addCrewConsumablesDemandModel);
		demandsButtonPanel.add(btnAddDemandModel);
		
		btnEditDemandModel = new JButton("Edit", new ImageIcon(getClass().getClassLoader().getResource("icons/comment_edit.png")));
		btnEditDemandModel.setMargin(new Insets(3,3,3,3));
		btnEditDemandModel.setToolTipText("Edit Demand Model");
		btnEditDemandModel.setEnabled(false);
		btnEditDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				DemandModelDialog.createAndShowGUI(missionsTab, (I_DemandModel)demandModelsList.getSelectedValue());
			}
		});
		demandsButtonPanel.add(btnEditDemandModel);
		
		btnRemoveDemandModel = new JButton("Remove", new ImageIcon(getClass().getClassLoader().getResource("icons/comment_delete.png")));
		btnRemoveDemandModel.setMargin(new Insets(3,3,3,3));
		btnRemoveDemandModel.setToolTipText("Remove Demand Model");
		btnRemoveDemandModel.setEnabled(false);
		btnRemoveDemandModel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mission.getDemandModels().remove(demandModelsList.getSelectedValue());
				updateView();
			}
		});
		demandsButtonPanel.add(btnRemoveDemandModel);
		add(demandsButtonPanel, c);
		
		c.gridy+=2;
		c.weighty = 0.9;
		c.fill = GridBagConstraints.BOTH;
		eventsTable = new EventTable(missionsTab);
		eventsTable.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if(e.getClickCount()==2 && eventsTable.getSelectedRow()>=0) {
					EventDialog.createAndShowGUI(missionsTab, eventsTable.getEvent(eventsTable.getSelectedRow()));
				}
			}
		});
		eventsTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				updateButtons();
			}
		});
		JScrollPane eventScroll = new JScrollPane(eventsTable);
		eventScroll.setBackground(Color.WHITE);
		eventScroll.getViewport().setBackground(Color.WHITE);
		eventScroll.setPreferredSize(new Dimension(150,150));
		add(eventScroll, c);
		
		JPanel eventButtonPanel = new JPanel();
		eventButtonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 3, 0));
		DropDownButton btnAddEvent = new DropDownButton("Add Event", new ImageIcon(getClass().getClassLoader().getResource("icons/cog_add.png")));
		btnAddEvent.getButton().setMargin(new Insets(3,3,3,3));
		btnAddEvent.setToolTipText("Add Event");
		JMenuItem addCreateEvent = new JMenuItem(EventType.CREATE.getName(), EventType.CREATE.getIcon());
		addCreateEvent.setToolTipText("Instantiate elements at a node or nested within a carrier element");
		addCreateEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.CREATE);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addCreateEvent);
		JMenuItem addAddEvent = new JMenuItem(EventType.ADD.getName(), EventType.ADD.getIcon());
		addAddEvent.setToolTipText("Add resources to a container element");
		addAddEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.ADD);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addAddEvent);
		JMenuItem addMoveEvent = new JMenuItem(EventType.MOVE.getName(), EventType.MOVE.getIcon());
		addMoveEvent.setToolTipText("Move elements to a node or to a carrier element");
		addMoveEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.MOVE);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addMoveEvent);
		JMenuItem addTransferEvent = new JMenuItem(EventType.TRANSFER.getName(), EventType.TRANSFER.getIcon());
		addTransferEvent.setToolTipText("Transfer resources from one container to another");
		addTransferEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.TRANSFER);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addTransferEvent);
		JMenuItem addRemoveEvent = new JMenuItem(EventType.REMOVE.getName(), EventType.REMOVE.getIcon());
		addRemoveEvent.setToolTipText("Remove elements from the simulation.");
		addRemoveEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.REMOVE);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addRemoveEvent);
		JMenuItem addReconfigureEvent = new JMenuItem(EventType.RECONFIGURE.getName(), EventType.RECONFIGURE.getIcon());
		addReconfigureEvent.setToolTipText("Reconfigure an element to a new state");
		addReconfigureEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.RECONFIGURE);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addReconfigureEvent);
		JMenuItem addReconfigureGroupEvent = new JMenuItem(EventType.RECONFIGURE_GROUP.getName(), EventType.RECONFIGURE_GROUP.getIcon());
		addReconfigureGroupEvent.setToolTipText("Reconfigure a set of elements to a new state type");
		addReconfigureGroupEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.RECONFIGURE_GROUP);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addReconfigureGroupEvent);
		JMenuItem addDemandEvent = new JMenuItem(EventType.DEMAND.getName(), EventType.DEMAND.getIcon());
		addDemandEvent.setToolTipText("Demand for resources by an element");
		addDemandEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.DEMAND);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addDemandEvent);
		JMenuItem addBurnEvent = new JMenuItem(EventType.BURN.getName(), EventType.BURN.getIcon());
		addBurnEvent.setToolTipText("Impulsive OMS or RCS burn");
		addBurnEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.BURN);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addBurnEvent);
		JMenuItem addEvaEvent = new JMenuItem(EventType.EVA.getName(), EventType.EVA.getIcon());
		addEvaEvent.setToolTipText("Extra-vehicular activity");
		addEvaEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.EVA);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addEvaEvent);
		JMenuItem addSpaceTransport = new JMenuItem(EventType.SPACE_TRANSPORT.getName(), EventType.SPACE_TRANSPORT.getIcon());
		addSpaceTransport.setToolTipText("Transports a stack from node to node along a trajectory of impulsive burns");
		addSpaceTransport.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.SPACE_TRANSPORT);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addSpaceTransport);
		JMenuItem addSurfaceTransport = new JMenuItem(EventType.SURFACE_TRANSPORT.getName(), EventType.SURFACE_TRANSPORT.getIcon());
		addSurfaceTransport.setToolTipText("Transports a surface vehicle from surface node to surface node");
		addSurfaceTransport.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.SURFACE_TRANSPORT);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addSurfaceTransport);
		JMenuItem addFlightProcess = new JMenuItem(EventType.FLIGHT_TRANSPORT.getName(), EventType.FLIGHT_TRANSPORT.getIcon());
		addFlightProcess.setToolTipText("Transports a stack from node to node under pre-specified flight constraints");
		addFlightProcess.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.FLIGHT_TRANSPORT);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addFlightProcess);
		JMenuItem addExplorationProcess = new JMenuItem(EventType.EXPLORATION.getName(), EventType.EXPLORATION.getIcon());
		addExplorationProcess.setToolTipText("Schedules EVAs at a specified frequency");
		addExplorationProcess.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				I_Event event = EventFactory.createEvent(EventType.EXPLORATION);
				prefill(event);
				EventDialog.createAndShowGUI(missionsTab, event);
			}
		});
		btnAddEvent.getMenu().add(addExplorationProcess);
		eventButtonPanel.add(btnAddEvent);
		
		btnEditEvent = new JButton("Edit", new ImageIcon(getClass().getClassLoader().getResource("icons/cog_edit.png")));
		btnEditEvent.setMargin(new Insets(3,3,3,3));
		btnEditEvent.setToolTipText("Edit Event");
		btnEditEvent.setEnabled(false);
		btnEditEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EventDialog.createAndShowGUI(missionsTab, eventsTable.getEvent(eventsTable.getSelectedRow()));
			}
		});
		eventButtonPanel.add(btnEditEvent);
		
		btnRemoveEvent = new JButton("Remove", new ImageIcon(getClass().getClassLoader().getResource("icons/cog_delete.png")));
		btnRemoveEvent.setMargin(new Insets(3,3,3,3));
		btnRemoveEvent.setToolTipText("Remove Event");
		btnRemoveEvent.setEnabled(false);
		btnRemoveEvent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				ArrayList<I_Event> events = new ArrayList<I_Event>();
				for(int row : eventsTable.getSelectedRows()) {
					events.add(eventsTable.getEvent(row));
				}
				JPanel warningPanel = new JPanel();
				warningPanel.setLayout(new GridBagLayout());
				GridBagConstraints c = new GridBagConstraints();
				c.insets = new Insets(2,2,2,2);
				c.gridx = 0;
				c.gridy = 0;
				c.weightx = 0;
				c.weighty = 0;
				c.anchor = GridBagConstraints.LINE_START;
				c.fill = GridBagConstraints.BOTH;
				warningPanel.add(new JLabel("Permanently delete the following events?"), c);
				c.gridy++;
				c.weighty = 1;
				DefaultListModel eventsList = new DefaultListModel();
				for(I_Event v : events) {
					eventsList.addElement(v);
				}
				warningPanel.add(new JScrollPane(new JList(eventsList)), c);
				int answer = JOptionPane.showOptionDialog(missionsTab, 
						warningPanel, 
						"Warning", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, null, null);
			    if (answer == JOptionPane.YES_OPTION) {
			    	for(I_Event event : events) {
			    		mission.getEventList().remove(event);
					}
			    	
			    }
				missionsTab.updateView();
			}
		});
		eventButtonPanel.add(btnRemoveEvent);
		c.gridy++;
		c.weighty = 0;
		c.fill = GridBagConstraints.NONE;
		add(eventButtonPanel, c);
		c.gridy++;
		c.anchor = GridBagConstraints.LINE_END;
		JButton btnOk = new JButton("Save", new ImageIcon(getClass().getClassLoader().getResource("icons/application_go.png")));
		btnOk.setMargin(new Insets(3,3,3,3));
		btnOk.setToolTipText("Save Mission");
		btnOk.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				missionsTab.saveMission(mission);
			}
		});
		add(btnOk, c);
	}
	private void initialize() {
		txtName.setText(mission.getName());
		calStartDate.setDate(mission.getStartDate());
		Node origin = mission.getOrigin();
		Node destination = mission.getDestination();
		Node returnOrigin = mission.getReturnOrigin();
		Node returnDestination = mission.getReturnDestination();
		ddlOrigin.removeAllItems();
		ddlDestination.removeAllItems();
		ddlReturnOrigin.removeAllItems();
		ddlReturnDestination.removeAllItems();
		ddlReturnOrigin.addItem(null);
		ddlReturnDestination.addItem(null);
		for(Node n : SpaceNetFrame.getSingleton().getScenarioPanel().getScenario().getNetwork().getNodes()) {
			ddlOrigin.addItem(n);
			ddlDestination.addItem(n);
			ddlReturnOrigin.addItem(n);
			ddlReturnDestination.addItem(n);
		}
		ddlOrigin.setSelectedItem(origin);
		ddlDestination.setSelectedItem(destination);
		ddlReturnOrigin.setSelectedItem(returnOrigin);
		ddlReturnDestination.setSelectedItem(returnDestination);
		
		demandModelsModel.clear();
		for(I_DemandModel m : mission.getDemandModels()) {
			demandModelsModel.addElement(m);
		}
		eventsTable.setEvents(mission.getEventList());
	}
	private void prefill(I_Event event) {
		if(eventsTable.getRowCount() > 0) {
			I_Event prevEvent;
			if(eventsTable.getSelectedRowCount()>0
					&& eventsTable.getSelectedRow()<eventsTable.getRowCount()) {
				prevEvent = eventsTable.getEvent(eventsTable.getSelectedRow());
			} else {
				prevEvent = eventsTable.getEvent(eventsTable.getRowCount()-1);
			}
			if(prevEvent instanceof I_Transport) {
				event.setTime(prevEvent.getTime() + ((I_Transport)prevEvent).getDuration());
				event.setPriority(1);
				event.setLocation(((I_Transport)prevEvent).getDestination());
			} else if(prevEvent instanceof I_Process) {
				event.setTime(prevEvent.getTime() + ((I_Process)prevEvent).getDuration());
				event.setPriority(1);
				event.setLocation(prevEvent.getLocation());
			} else {
				event.setTime(prevEvent.getTime());
				event.setPriority(Math.min(prevEvent.getPriority() + 1, 5));
				event.setLocation(prevEvent.getLocation());
			}
			event.setName(mission.getName().substring(0,Math.min(mission.getName().length(), 5)) + " | " + event.getName());
		} else {
			event.setTime(0);
			event.setPriority(1);
			event.setName(mission.getName().substring(0,Math.min(mission.getName().length(), 5)) + " | " + event.getName());
		}
	}
	
	/**
	 * Update view.
	 */
	public void updateView() {
		Collections.sort(mission.getEventList());
		eventsTable.updateView();
		demandModelsModel.clear();
		for(I_DemandModel m : mission.getDemandModels()) {
			demandModelsModel.addElement(m);
		}
		updateButtons();
	}
	private void updateButtons() {
		if(demandModelsList.getSelectedIndices().length == 1) {
			btnEditDemandModel.setEnabled(true);
			btnRemoveDemandModel.setEnabled(true);
		} else if(demandModelsList.getSelectedIndices().length > 1) {
			btnEditDemandModel.setEnabled(false);
			btnRemoveDemandModel.setEnabled(true);
		} else {
			btnEditDemandModel.setEnabled(false);
			btnRemoveDemandModel.setEnabled(false);
		}
		if(eventsTable.getSelectedRowCount() == 1) {
			btnEditEvent.setEnabled(true);
			btnRemoveEvent.setEnabled(true);
		} else if(eventsTable.getSelectedRowCount() > 1) {
			btnEditEvent.setEnabled(false);
			btnRemoveEvent.setEnabled(true);
		} else {
			btnEditEvent.setEnabled(false);
			btnRemoveEvent.setEnabled(false);
		}
	}
	
	/**
	 * Gets the mission.
	 * 
	 * @return the mission
	 */
	public Mission getMission() {
		return mission;
	}
	
	/**
	 * Sets the mission.
	 * 
	 * @param mission the new mission
	 */
	public void setMission(Mission mission) {
		this.mission = mission;
		if(mission!=null) initialize();
	}
}