/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.model;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.element.PropulsiveVehicle;
import edu.mit.spacenet.domain.element.ResourceTank;
import edu.mit.spacenet.domain.element.SurfaceVehicle;
import edu.mit.spacenet.domain.model.BoilOffDemandModel;
import edu.mit.spacenet.gui.component.UnitsWrapper;

/**
 * The panel for viewing and editing boil off demand models.
 * 
 * @author Paul Grogan
 */
public class BoilOffDemandModelPanel extends AbstractDemandModelPanel {
	private static final long serialVersionUID = -966209133130231178L;
	
	private BoilOffDemandModel demandModel;
	
	private JComboBox ddlFuelTank;
	private JSpinner rateSpinner;
	private SpinnerNumberModel rateModel;
	
	/**
	 * Instantiates a new boil off demand model panel.
	 * 
	 * @param demandModelDialog the demand model dialog
	 * @param demandModel the demand model
	 */
	public BoilOffDemandModelPanel(DemandModelDialog demandModelDialog,
			BoilOffDemandModel demandModel) {
		super(demandModelDialog, demandModel);
		this.demandModel = demandModel;
		buildPanel();
		initialize();
	}
	private void buildPanel() {
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 0;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_END;
		c.fill = GridBagConstraints.NONE;
		add(new JLabel("Tank: "), c);
		c.gridy++;
		c.weighty = 1;
		c.anchor = GridBagConstraints.FIRST_LINE_END;
		add(new JLabel("Rate: "), c);
		
		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		ddlFuelTank = new JComboBox();
		add(ddlFuelTank, c);
		c.gridy++;
		c.weighty = 1;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		rateModel = new SpinnerNumberModel(0,0,100,.001);
		rateSpinner = new JSpinner(rateModel);
		rateSpinner.setPreferredSize(new Dimension(75,20));
		add(new UnitsWrapper(rateSpinner, "% per day"), c);
	}
	private void initialize() {
		I_Element element = getDemandModelDialog().getStateDialog().getElementDialog().getElement();
		if(element instanceof ResourceTank) {
			ddlFuelTank.addItem(element);
		} else if(element instanceof PropulsiveVehicle) {
			if(((PropulsiveVehicle)element).getOmsFuelTank() != null)
				ddlFuelTank.addItem(((PropulsiveVehicle)element).getOmsFuelTank());
			if(((PropulsiveVehicle)element).getRcsFuelTank() != null && 
					((PropulsiveVehicle)element).getOmsFuelTank() != ((PropulsiveVehicle)element).getRcsFuelTank())
				ddlFuelTank.addItem(((PropulsiveVehicle)element).getRcsFuelTank());
		} else if(element instanceof SurfaceVehicle) {
			if(((SurfaceVehicle)element).getFuelTank() != null) {
				ddlFuelTank.addItem(((SurfaceVehicle)element).getFuelTank());
			}
		}
		ddlFuelTank.setSelectedItem(demandModel.getTank());
		rateModel.setValue(demandModel.getRate()*100);
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.model.AbstractDemandModelPanel#getDemandModel()
	 */
	@Override
	public BoilOffDemandModel getDemandModel() {
		return demandModel;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.model.AbstractDemandModelPanel#saveDemandModel()
	 */
	@Override
	public void saveDemandModel() {
		demandModel.setTank((ResourceTank)ddlFuelTank.getSelectedItem());
		demandModel.setRate(rateModel.getNumber().doubleValue()/100d);
	}
}
