/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.model;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.DecimalFormat;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JTextField;

import edu.mit.spacenet.domain.ClassOfSupply;
import edu.mit.spacenet.domain.model.IsruProductionModel;
import edu.mit.spacenet.domain.resource.GenericResource;
import edu.mit.spacenet.domain.resource.I_Resource;
import edu.mit.spacenet.gui.SpaceNetFrame;
import edu.mit.spacenet.gui.component.UnitsWrapper;

/**
 * The panel for viewing and editing isru production models.
 * 
 * @author Paul Grogan
 */
public class IsruProductionModelPanel extends AbstractDemandModelPanel {
	private static final long serialVersionUID = -966209133130231178L;
	
	private IsruProductionModel demandModel;
	
	private JComboBox ddlElement, ddlResource;
	private JTextField txtProduction;
	
	/**
	 * Instantiates a new isru production model panel.
	 * 
	 * @param demandModelDialog the demand model dialog
	 * @param demandModel the demand model
	 */
	public IsruProductionModelPanel(DemandModelDialog demandModelDialog,
			IsruProductionModel demandModel) {
		super(demandModelDialog, demandModel);
		this.demandModel = demandModel;
		buildPanel();
		initialize();
	}
	private void buildPanel() {
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.LINE_END;
		add(new JLabel("Element: "), c);
		c.gridy++;
		add(new JLabel("Production: "), c);
		c.gridy++;
		c.weighty = 1;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		add(new JLabel("Resource: "), c);
		
		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 0;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		ddlElement = new JComboBox();
		ddlElement.setEnabled(false);
		add(ddlElement, c);
		c.gridy++;
		txtProduction = new JTextField(10);
		txtProduction.setHorizontalAlignment(JTextField.RIGHT);
		txtProduction.setEnabled(false);
		add(new UnitsWrapper(txtProduction, "kg / year"), c);
		c.gridy++;
		c.weighty = 1;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		ddlResource = new JComboBox();
		add(ddlResource, c);
	}
	private void initialize() {
		demandModel.setElement(getDemandModelDialog().getStateDialog().getElementDialog().getElement());
		ddlElement.addItem(demandModel.getElement());
		ddlElement.setSelectedItem(demandModel.getElement());
		
		DecimalFormat format = new DecimalFormat("0.0");
		txtProduction.setText(format.format(demandModel.getAnnualProduction()));
		
		ddlResource.addItem(new GenericResource(ClassOfSupply.COS2));
		ddlResource.addItem(new GenericResource(ClassOfSupply.COS203));
		for(I_Resource r : SpaceNetFrame.getSingleton().getScenarioPanel().getScenario().getDataSource().getResourceLibrary()) {
			if(r.getClassOfSupply().equals(ClassOfSupply.COS203))
				ddlResource.addItem(r);
		}
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.model.AbstractDemandModelPanel#getDemandModel()
	 */
	@Override
	public IsruProductionModel getDemandModel() {
		return demandModel;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.model.AbstractDemandModelPanel#saveDemandModel()
	 */
	@Override
	public void saveDemandModel() {
		demandModel.setResource((I_Resource)ddlResource.getSelectedItem());
	}
}
