/*
 * Copyright (c) 2009 MIT Strategic Engineering Research Group
 * 
 * This file is part of SpaceNet 2.5.
 * 
 * SpaceNet 2.5 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SpaceNet 2.5 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SpaceNet 2.5.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.mit.spacenet.gui.model;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import edu.mit.spacenet.domain.element.I_Element;
import edu.mit.spacenet.domain.model.SparingByMassDemandModel;
import edu.mit.spacenet.gui.component.UnitsWrapper;

/**
 * The panel for viewing and editing sparing by mass demand models.
 * 
 * @author Paul Grogan
 */
public class SparingByMassDemandModelPanel extends AbstractDemandModelPanel {
	private static final long serialVersionUID = -966209133130231178L;
	
	private SparingByMassDemandModel demandModel;
	
	private JComboBox ddlElement;
	private SpinnerNumberModel unpressRateModel, pressRateModel;
	private JSpinner unpressRateSpinner, pressRateSpinner;
	
	/**
	 * Instantiates a new sparing by mass demand model panel.
	 * 
	 * @param demandModelDialog the demand model dialog
	 * @param demandModel the demand model
	 */
	public SparingByMassDemandModelPanel(DemandModelDialog demandModelDialog,
			SparingByMassDemandModel demandModel) {
		super(demandModelDialog, demandModel);
		this.demandModel = demandModel;
		buildPanel();
		initialize();
	}
	private void buildPanel() {
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.LINE_END;
		add(new JLabel("Element: "), c);
		c.gridy++;
		add(new JLabel("Unpress. Rate: "), c);
		c.gridy++;
		c.weighty = 1;
		c.anchor = GridBagConstraints.FIRST_LINE_END;
		add(new JLabel("Press. Rate: "), c);
		
		c.gridx = 1;
		c.gridy = 0;
		c.weighty = 0;
		c.weightx = 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		ddlElement = new JComboBox();
		ddlElement.setEnabled(false);
		add(ddlElement, c);
		c.gridy++;
		c.fill = GridBagConstraints.NONE;
		unpressRateModel = new SpinnerNumberModel(0.00,0.00,1000,.5);
		unpressRateSpinner = new JSpinner(unpressRateModel);
		add(new UnitsWrapper(unpressRateSpinner, "% mass / year"), c);
		c.gridy++;
		c.weighty = 1;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		pressRateModel = new SpinnerNumberModel(0.00,0.00,1000,.5);
		pressRateSpinner = new JSpinner(pressRateModel);
		add(new UnitsWrapper(pressRateSpinner, "% mass / year"), c);
	}
	private void initialize() {
		demandModel.setElement(getDemandModelDialog().getStateDialog().getElementDialog().getElement());
		ddlElement.addItem(getDemandModelDialog().getStateDialog().getElementDialog().getElement());
		ddlElement.setSelectedItem(getDemandModelDialog().getStateDialog().getElementDialog().getElement());
		
		unpressRateModel.setValue(demandModel.getUnpressurizedSparesRate()*100);
		pressRateModel.setValue(demandModel.getPressurizedSparesRate()*100);
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.model.AbstractDemandModelPanel#getDemandModel()
	 */
	@Override
	public SparingByMassDemandModel getDemandModel() {
		return demandModel;
	}
	
	/* (non-Javadoc)
	 * @see edu.mit.spacenet.gui.model.AbstractDemandModelPanel#saveDemandModel()
	 */
	@Override
	public void saveDemandModel() {
		demandModel.setElement((I_Element)ddlElement.getSelectedItem());
		demandModel.setUnpressurizedSparesRate(unpressRateModel.getNumber().doubleValue()/100d);
		demandModel.setPressurizedSparesRate(pressRateModel.getNumber().doubleValue()/100d);
	}
}
